#............................................
## Map makin' from 2017.11.1 - Requires county_data, etc. files to be loaded ####
#............................................
# Add tmap:: library(sp) #for spatial objects
library(sp)
library(rgdal) #for reading and writing shapefiles
library(rgeos) #for spatial tools like gCentroid
library(RColorBrewer) #For better, much better color ramps
#library(maps) # http://dmcglinn.github.io/quant_methods/lessons/shapefiles_and_rasters.html
library(tmap)

head(county_data) #formatter[formatter$variable=="cores",]

getwd()
if(!file.exists("./maps/nc counties.shp")){ #Run Once to get NC Counties.
  dir.create("./maps/")
  # ftp://ftp2.census.gov/geo/tiger/TIGER2016/ or 
  # https://www.census.gov/cgi-bin/geo/shapefiles/index.php
  download.file("ftp://ftp2.census.gov/geo/tiger/TIGER2015/COUNTY/tl_2015_us_county.zip", "./maps/uscounties.zip")
  unzip("./maps/uscounties.zip", exdir="./maps")
  dir("./maps/")
  us_counties = readOGR(dsn="./maps", layer="tl_2015_us_county", stringsAsFactors = F)
  plot(us_counties)
  nc_counties = us_counties[us_counties$STATEFP == 37,] #look how slick this is.
  writeOGR(nc_counties, dsn="./maps", layer="nc counties", driver="ESRI Shapefile", overwrite_layer = T)
}
nc_counties = readOGR(dsn="./maps", layer="nc counties", stringsAsFactors = F)
class(nc_counties)
plot(nc_counties)
str(nc_counties, max.level = 2)
# ... or us tigris package. Also see acs package

#remember to merge with the spatial object, not just the data slot, to keep it from reordering
nc_counties = merge(nc_counties, county_data, by.x="GEOID", by.y="helper")

spplot(nc_counties, "pct_pnc5") # on spplot: https://edzer.github.io/sp/
spplot(nc_counties, c("pct_pnc5", "pct_preterm")) # on spplot: https://edzer.github.io/sp/
writeOGR(nc_counties, dsn="./maps", layer="nc counties w data", driver="ESRI Shapefile", overwrite_layer = T)

sum(nc_counties@data$n)
sum(nc_counties[nc_counties$NAME %in% c("Orange", "Durham", "Chatham"),]$n)


### Using base plot()
display.brewer.all()
nc_counties$pct_preterm_colors = as.character(cut(nc_counties$pct_preterm, 10, labels = brewer.pal(10,"RdYlGn")))
plot(nc_counties, co=nc_counties$pct_preterm_colors, main="% Preterm birth")
county_centroids = gCentroid(nc_counties, byid = T)
text(county_centroids@coords, labels=as.character(nc_counties$NAME), cex=0.4)

### Using spplot()
spplot(nc_counties, "pct_pnc5") #ugh. gorgeous
nc_counties$pct_pnc5_f = cut(nc_counties$pct_pnc5, 5)
spplot(nc_counties, "pct_pnc5_f", 
       col.regions=brewer.pal(5,"RdYlGn"), 
       main="% of Births with Early Start of Prenatal Care (<5mo)")

### Using ggplot() - nice, but slow
names(nc_counties@data)
nc_counties_fort = fortify(nc_counties, region = "GEOID")
str(nc_counties_fort)
nc_counties_fort = merge(nc_counties_fort, nc_counties, by.x="id", by.y="GEOID")
#nc.counties.fort = nc.counties.fort[order(nc.counties.fort$order),]
names(nc_counties_fort)
library(ggthemes) # for theme_map
cut(nc_counties$pct_pnc5, 5)
pretty(nc_counties$pct_pnc5) #funny name
nc_counties_fort$pct_pnc5_f = cut(nc_counties_fort$pct_pnc5, breaks=pretty(pretty(nc_counties_fort$pct_pnc5)))
table(nc_counties_fort$pct_pnc5_f)
#grep to make , a -_ move to top, like 
#http://time_com/4394141/zika-abortion-usa/
mymap = 
  ggplot(nc_counties_fort, aes(x=long, y=lat, group=group, fill=pct_pnc5_f)) + 
  geom_polygon(color="white")+coord_map()+
  labs(title=paste0("Early prenatal care by county in North Carolina,", births$byear[1]), 
       subtitle="This ggplot is gorgeous, holy cow", 
       x="", y="")+
  scale_fill_brewer(name="% Early Prenatal Care", type="seq", palette ="Greens")+
  #scale_fill_gradient(name="% Early Prenatal Care", low = "white", high = "dark green")+
  theme_map()+theme(legend.position="right")
#theme_map()+theme(legend_position="bottom")+guides(fill=guide_legend(nrow=1))#right")
mymap 
# http://novyden.blogspot.com/2013/09/how-to-expand-color-palette-with-ggplot.html
mymap+theme_minimal() #also a nice one / clean. 

# tmap
tm_shape(nc_counties)+
  tm_fill("pct_pnc5", title="% Early PNC")+
  tm_borders()+ #could add more layers with tm_shape and then tm_whatever...
  tm_layout(main.title="% Early Prenatal Care by County")

# sf
library(sf)
nc_counties_sf = sf::read_sf(dsn="./maps", layer="nc counties", stringsAsFactors = F)
str(nc_counties_sf)
plot(nc_counties_sf)

# Demos: transform, gTouches,
proj4string(nc_counties) #or nc_counties@proj4string  
nc_stateplane_proj = "+proj=lcc +lat_1=36.16666666666666 +lat_2=34.33333333333334 +lat_0=33.75 +lon_0=-79 +x_0=609601.2192024384 +y_0=0 +ellps=GRS80 +datum=NAD83 +to_meter=0.3048006096012192 +no_defs"
nc_counties_stateplane = spTransform(nc_counties, nc_stateplane_proj)

county_cents = gCentroid(nc_counties_stateplane, byid=T) #just the spatial object
county_cents_sp = SpatialPointsDataFrame(county_cents, data=nc_counties@data) #need to add back the data

plot(nc_counties_stateplane); plot(county_cents, add=T)
plot(county_cents_sp[county_cents_sp$NAME == "Orange",], col="blue", add=T, pch="0")
plot(gBuffer(county_cents, width = 10*5280, byid = T), add=T) #10 mi buffers

plot(nc_counties[county_cents_sp$NAME == "Orange",])
orange_neighbors = gTouches(nc_counties[county_cents_sp$NAME == "Orange",], nc_counties, byid = T)
str(orange_neighbors) # note it's a matrix
plot(nc_counties); plot(nc_counties[orange_neighbors[,1],], col="blue", add=T)


# make_EPSG() # or lookup

###############################################################################
# Notes
# http://www.kevjohnson.org/making-maps-in-r/
# https://www.google.com/search?q=north+carolina+counties+table&tbm=isch&tbo=u&source=univ&sa=X&ved=0ahUKEwiLnYyDxNbNAhVQzGMKHUAUD4kQsAQIHQ&biw=1536&bih=716
###############################################################################